@@ -1,47 +1,34 @@
+import os
 import cgi
 import asyncio
 import hashlib
 import pathlib
+import warnings
+from pathlib import Path
 from itertools import count
+from concurrent.futures import ThreadPoolExecutor
+
+import aiohttp
 
 import parfive
 
-__all__ = ['run_in_thread', 'Token', 'FailedDownload', 'default_name',
-           'in_notebook']
+__all__ = [
+    "cancel_task",
+    "run_in_thread",
+    "Token",
+    "FailedDownload",
+    "default_name",
+    "remove_file",
+]
 
 
-def in_notebook():
-    try:
-        import ipykernel.zmqshell
-        shell = get_ipython()  # noqa
-        if isinstance(shell, ipykernel.zmqshell.ZMQInteractiveShell):
-            try:
-                # Newer tqdm
-                import tqdm.notebook
-                return tqdm.notebook.IPY > 0
-            except ImportError:
-                # Older tqdm
-                try:
-                    # Check that we can import the right widget
-                    from tqdm import _tqdm_notebook
-                    _tqdm_notebook.IntProgress
-                except Exception:
-                    return False
-            except Exception:
-                return False
-            return True
-        return False
-    except Exception:
-        return False
-
-
-def default_name(path, resp, url):
-    url_filename = url.split('/')[-1]
+def default_name(path: os.PathLike, resp: aiohttp.ClientResponse, url: str) -> os.PathLike:
+    url_filename = url.split("/")[-1]
     if resp:
         cdheader = resp.headers.get("Content-Disposition", None)
         if cdheader:
             value, params = cgi.parse_header(cdheader)
-            name = params.get('filename', url_filename)
+            name = params.get("filename", url_filename)
         else:
             name = url_filename
     else:
@@ -49,7 +36,7 @@ def default_name(path, resp, url):
     return pathlib.Path(path) / name
 
 
-def run_in_thread(aio_pool, loop, coro):
+def run_task_in_thread(loop, coro):
     """
     This function returns the asyncio Future after running the loop in a
     thread.
@@ -57,7 +44,12 @@ def run_in_thread(aio_pool, loop, coro):
     This makes the return value of this function the same as the return
     of ``loop.run_until_complete``.
     """
-    return aio_pool.submit(loop.run_until_complete, coro).result()
+    with ThreadPoolExecutor(max_workers=1) as aio_pool:
+        try:
+            future = aio_pool.submit(loop.run_until_complete, coro)
+        except KeyboardInterrupt:
+            future.cancel()
+    return future.result()
 
 
 async def get_ftp_size(client, filepath):
@@ -89,7 +81,7 @@ def replacement_filename(path):
     if not path.exists:
         return path
 
-    suffix = ''.join(path.suffixes)
+    suffix = "".join(path.suffixes)
     for c in count(1):
         if suffix:
             name, _ = path.name.split(suffix)
@@ -113,7 +105,7 @@ def get_filepath(filepath, overwrite):
     if filepath.exists():
         if not overwrite:
             return str(filepath), True
-        if overwrite == 'unique':
+        if overwrite == "unique":
             filepath = replacement_filename(filepath)
     if not filepath.parent.exists():
         filepath.parent.mkdir(parents=True)
@@ -128,12 +120,17 @@ def sha256sum(filename):
     h = hashlib.sha256()
     b = bytearray(128 * 1024)
     mv = memoryview(b)
-    with open(filename, 'rb', buffering=0) as f:
+    with open(filename, "rb", buffering=0) as f:
         for n in iter(lambda: f.readinto(mv), 0):
             h.update(mv[:n])
     return h.hexdigest()
 
 
+class MultiPartDownloadError(Exception):
+    def __init__(self, response):
+        self.response = response
+
+
 class FailedDownload(Exception):
     def __init__(self, filepath_partial, url, exception):
         self.filepath_partial = filepath_partial
@@ -143,7 +140,7 @@ class FailedDownload(Exception):
 
     def __repr__(self):
         out = super().__repr__()
-        out += f'\n {self.url} {self.exception}'
+        out += f"\n {self.url} {self.exception}"
         return out
 
     def __str__(self):
@@ -175,3 +172,45 @@ class _QueueList(list):
             queue.put_nowait(item)
         self.clear()
         return queue
+
+
+class ParfiveUserWarning(UserWarning):
+    """
+    Raised for not-quite errors.
+    """
+
+
+class ParfiveFutureWarning(FutureWarning):
+    """
+    Raised for future changes to the parfive API.
+    """
+
+
+def remove_file(filepath):
+    """
+    Remove the file from the disk, if it exists
+    """
+    filepath = Path(filepath)
+    try:
+        # When we drop 3.7 support we can use unlink(missing_ok=True)
+        if filepath.exists():
+            filepath.unlink()
+    except Exception as remove_exception:
+        warnings.warn(
+            f"Failed to delete possibly incomplete file {filepath} {remove_exception}",
+            ParfiveUserWarning,
+        )
+
+
+async def cancel_task(task):
+    """
+    Call cancel on a task and then wait for it to exit.
+
+    Return True if the task was cancelled, False otherwise.
+    """
+    task.cancel()
+    try:
+        await task
+    except asyncio.CancelledError:
+        return True
+    return task.cancelled()
